﻿// <copyright file="EquipmentDictionaryWorkbookBehavior.cs" company="PEER Group GmbH">
// Copyright (c) PEER Group GmbH. All rights reserved.
// </copyright>

using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;
using Infragistics.Controls.Grids;
using Infragistics.Documents.Excel;
using SpreadsheetOwnerErrorSampleV3.Interfaces;
using SpreadsheetOwnerErrorSampleV3.Views;

namespace SpreadsheetOwnerErrorSampleV3.Core.Behaviors
{
	/// <summary>
	/// Contains logic of the <see cref="EquipmentDictionaryEditorView"/> control.
	/// The logic of the control is extracted into this behavior in order to make custom properties
	/// bindable to view model properties.
	/// </summary>
	public class EquipmentDictionaryWorkbookBehavior : Behavior<EditorViewWithSpreadsheet>
	{
		private static int sIndex = 0;
		private LogNotify mLog;

		//private SpreadsheetCommands mCommands;
		private SpreadsheetCell mActiveCell;
		//private UndoHistoryWrapper mUndoHistory;
		private object mActiveCellValue;
		private bool mUndoItemNeeded;

		private XamSpreadsheet Spreadsheet => AssociatedObject.SpreadSheet;

		private Workbook Workbook => Spreadsheet.Workbook;

		public EquipmentDictionaryWorkbookBehavior()
		{
			mLog = new LogNotify($"BEH.{++sIndex}");
			mLog.Write($"** Created behavior: {CanFreeze}");
		}

		/// <summary>Gets or sets a value indicating whether the control data are modified.</summary>
		public bool IsSpreadsheetDirty
		{
			get => (bool)GetValue(IsSpreadsheetDirtyProperty);
			set => SetValue(IsSpreadsheetDirtyProperty, value);
		}

		/// <summary>Dependency property defined for the <see cref="IsSpreadsheetDirty"/> property.</summary>
		public static readonly DependencyProperty IsSpreadsheetDirtyProperty =
			DependencyProperty.Register("IsSpreadsheetDirty", typeof(bool),
				typeof(EquipmentDictionaryWorkbookBehavior), new PropertyMetadata(false));

		/// <summary>Gets or sets control data in the workbook representation.</summary>
		public IWorkbook WorkbookData
		{
			get => (IWorkbook)GetValue(WorkbookDataProperty);
			set
			{
				SetValue(WorkbookDataProperty, value);
				mLog.Write(Spreadsheet, "Apply WorkbookData");
				value.ApplyTo(Spreadsheet);
			}
		}

		/// <summary>Dependency property defined for <see cref="WorkbookData"/> property.</summary>
		public static readonly DependencyProperty WorkbookDataProperty =
			DependencyProperty.Register("WorkbookData", typeof(IWorkbook),
				typeof(EquipmentDictionaryWorkbookBehavior), new PropertyMetadata(WorkbookDataChangedCallback));

		/// <summary>Gets or sets a value indicating whether the spreadsheet is editable.</summary>
		public bool IsReadOnly
		{
			get => (bool)GetValue(IsReadOnlyProperty);
			set => SetValue(IsReadOnlyProperty, value);
		}

		/// <summary>Dependency property defined for <see cref="IsReadOnly"/> property.</summary>
		public static readonly DependencyProperty IsReadOnlyProperty =
			DependencyProperty.Register("IsReadOnly", typeof(bool),
				typeof(EquipmentDictionaryWorkbookBehavior), new PropertyMetadata(false, IsReadOnlyChangedCallback));

		/// <summary>Gets or sets a value indicating the Worksheets name.</summary>
		public string ActiveWorksheetName
		{
			get => (string)GetValue(ActiveWorksheetNameProperty);
			set => SetValue(ActiveWorksheetNameProperty, value);
		}

		/// <summary>Dependency property defined for <see cref="ActiveWorksheetName"/> property.</summary>
		public static readonly DependencyProperty ActiveWorksheetNameProperty =
			DependencyProperty.Register("ActiveWorksheetName", typeof(string),
				typeof(EquipmentDictionaryWorkbookBehavior),
				new PropertyMetadata(string.Empty));

		/// <summary>Gets or sets events fired when worksheet is added.</summary>
		public IAddWorksheetEvents AddWorksheetEvents
		{
			get => (IAddWorksheetEvents)GetValue(AddWorksheetEventsProperty);
			set => SetValue(ActiveWorksheetNameProperty, value);
		}

		/// <summary>Dependency property defined for <see cref=""/> property.</summary>
		public static readonly DependencyProperty AddWorksheetEventsProperty =
			DependencyProperty.Register("AddWorksheetEvents", typeof(IAddWorksheetEvents),
				typeof(EquipmentDictionaryWorkbookBehavior), new PropertyMetadata(AddWorksheetEventsChangedCallback));

		/// <inheritdoc cref="Behavior"/>
		protected override void OnAttached()
		{
			base.OnAttached();
			AssociatedObject.Initialized += AssociatedObjectOnInitialized;
		}

		/// <inheritdoc cref="Behavior"/>
		protected override void OnDetaching()
		{
			Spreadsheet.EditModeEntering -= SpreadSheetOnEditModeEntering;
			//Spreadsheet.CommandExecuting -= SpreadSheetOnCommandExecuting;
			//Spreadsheet.CommandExecuted -= SpreadSheetOnCommandExecuted;

			//if (Application.Current is IApplicationState { IsReadOnlyMode: true })
			//	return;

			Spreadsheet.EditModeExited -= SpreadSheetOnEditModeExited;
			Spreadsheet.ActiveWorksheetChanged -= SpreadSheetOnActiveWorksheetChanged;
			Spreadsheet.ActiveCellChanged -= SpreadSheetOnActiveCellChanged;
			Spreadsheet.EditModeExiting -= SpreadsheetOnEditModeExiting;
			Spreadsheet.WorkbookDirtied -= SpreadSheetOnWorkbookDirtied;
			AssociatedObject.Initialized -= AssociatedObjectOnInitialized;
			base.OnDetaching();
		}

		private static void IsReadOnlyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var isReadOnly = (bool)e.NewValue;
			var workbook = ((EquipmentDictionaryWorkbookBehavior)d).Workbook;

			ProtectWorkbook(workbook, isReadOnly);
		}

		private static void ProtectWorkbook(Workbook workbook, bool isReadOnly)
		{
			var isEditable = !isReadOnly;

			foreach (var sheet in workbook.Worksheets)
			{
				sheet.Unprotect();
				sheet.Protect(allowInsertingColumns: false,
					allowDeletingColumns: false,
					allowInsertingRows: isEditable,
					allowDeletingRows: isEditable,
					allowInsertingHyperlinks: isEditable,
					allowEditObjects: isEditable,
					allowUsingPivotTables: false,
					allowEditScenarios: false,
					allowFiltering: true,
					allowFormattingCells: false,
					allowFormattingColumns: true,
					allowFormattingRows: true,
					allowSorting: true);

				if (isReadOnly)
					sheet.DataValidationRules.Clear();
			}
		}

		private static void WorkbookDataChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var behaviorInstance = d as EquipmentDictionaryWorkbookBehavior;
			(e.NewValue as IWorkbook)?.ApplyTo(behaviorInstance?.Spreadsheet);

			if (behaviorInstance == null)
				return;

			var spreadsheet = behaviorInstance.Spreadsheet;
			if (spreadsheet.Workbook.Worksheets.Exists(behaviorInstance.ActiveWorksheetName))
			{
				spreadsheet.ActiveWorksheet = spreadsheet.Workbook.Worksheets[behaviorInstance.ActiveWorksheetName];
				spreadsheet.ActiveCell = behaviorInstance.mActiveCell;
			}

			behaviorInstance.mActiveCellValue = behaviorInstance.GetCellValue(spreadsheet.ActiveCell);

			ProtectWorkbook(behaviorInstance.Workbook, behaviorInstance.IsReadOnly);
			AutoFitColumnWidthIn(behaviorInstance.Workbook);
			spreadsheet.UndoManager.ClearHistory();
		}

		private static void AddWorksheetEventsChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			if (!(d is EquipmentDictionaryWorkbookBehavior behaviorInstance))
				return;

			//if (e.OldValue is IAddWorksheetEvents oldEvents)
			//	oldEvents.NewWorksheetAdding -= behaviorInstance.OnNewWorksheetAdding;

			//if (e.NewValue is IAddWorksheetEvents newEvents)
			//	newEvents.NewWorksheetAdding += behaviorInstance.OnNewWorksheetAdding;
		}

		private static void AutoFitColumnWidthIn(Workbook workbook)
		{
			foreach (var item in workbook.Worksheets)
			{
				AutoFitColumnWidthIn(item);
			}
		}

		private static void AutoFitColumnWidthIn(Worksheet worksheet)
		{
			foreach (var cell in worksheet.Rows[0].Cells)
			{
				var columnIndex = cell.ColumnIndex;
				var autoFitWidth = worksheet.Columns[columnIndex].CalculateAutoFitWidth();
				if (autoFitWidth < worksheet.Columns[columnIndex].Width)
					worksheet.Columns[columnIndex].Width = autoFitWidth;
			}
		}

		private void AssociatedObjectOnInitialized(object sender, EventArgs e)
		{
			//var undoHistory = new UndoItems(Spreadsheet.UndoManager);
			//var redoHistory = new RedoItems(Spreadsheet.UndoManager);
			//mUndoHistory = new UndoHistoryWrapper(undoHistory, redoHistory);
			//mCommands = new SpreadsheetCommands(Spreadsheet, undoHistory, redoHistory);

			Spreadsheet.EditModeEntering += SpreadSheetOnEditModeEntering;
			//Spreadsheet.CommandExecuting += SpreadSheetOnCommandExecuting;
			//Spreadsheet.CommandExecuted += SpreadSheetOnCommandExecuted;

			//if (Application.Current is IApplicationState { IsReadOnlyMode: true })
			//	return;

			Spreadsheet.EditModeEntered += SpreadsheetOnEditModeEntered;
			Spreadsheet.EditModeExited += SpreadSheetOnEditModeExited;
			Spreadsheet.ActiveWorksheetChanged += SpreadSheetOnActiveWorksheetChanged;
			Spreadsheet.ActiveCellChanged += SpreadSheetOnActiveCellChanged;
			Spreadsheet.EditModeExiting += SpreadsheetOnEditModeExiting;
			Spreadsheet.WorkbookDirtied += SpreadSheetOnWorkbookDirtied;

			ActiveWorksheetName = Spreadsheet.ActiveWorksheet.Name;
		}

		private void SpreadSheetOnWorkbookDirtied(object sender, SpreadsheetWorkbookDirtiedEventArgs e)
		{
			var activeCellInstance = Spreadsheet.ActiveWorksheet.Rows[mActiveCell.Row]
				.Cells[mActiveCell.Column];

			if (!IsDropdownCell(activeCellInstance) || Spreadsheet.IsInEditMode)
				return;

			var newCellValue = activeCellInstance.Value;
			if (mActiveCellValue?.ToString() == newCellValue?.ToString())
				return;

			CreateRowIfNotExists(ActiveWorksheetName, mActiveCell.Row);
			mActiveCellValue = newCellValue;

			//AddDropdownCellUpdateIntoHistory();
			SetSpreadsheetDirty();
		}

		private void OnNewWorksheetAdding(object sender, EventArgs args)
		{
			//var transaction = new NonHistoryTransaction(Spreadsheet.UndoManager);
			//transaction.Start();

			//void CloseTransaction(object sender, EventArgs args)
			//{
			//	transaction.Commit();
			//	AddWorksheetEvents.NewWorksheetAdded -= CloseTransaction;
			//}

			//AddWorksheetEvents.NewWorksheetAdded += CloseTransaction;
		}

		private void SpreadsheetOnEditModeExiting(object sender, SpreadsheetEditModeExitingEventArgs e)
		{
			if (IsSpreadsheetDirty || !e.AcceptChanges
				|| IsDropdownCell(Spreadsheet.ActiveWorksheet.Rows[e.Cell.Row].Cells[e.Cell.Column]))
				return;

			var oldCellValue = Spreadsheet.ActiveWorksheet.Rows[e.Cell.Row].Cells[e.Cell.Column]
				.Value?.ToString() ?? string.Empty;

			var newCellValue = e.EditText ?? string.Empty;
			if (oldCellValue == newCellValue)
				return;

			CreateRowIfNotExists(ActiveWorksheetName, e.Cell.Row);
			SetSpreadsheetDirty();
			mUndoItemNeeded = true;
		}

		private void SpreadSheetOnActiveCellChanged(object sender, SpreadsheetActiveCellChangedEventArgs e)
		{
			mActiveCell = e.NewValue;
			mActiveCellValue = GetCellValue(e.NewValue);
		}

		//private void SpreadSheetOnCommandExecuting(object sender, SpreadsheetCommandExecutingEventArgs e)
		//{
		//	// ReSharper disable once AssignmentInConditionalExpression
		//	if (e.Cancel = IsReadOnly && mCommands.CancelInReadOnly(e.Command))
		//		return;

		//	if (mCommands.OnCommandExecuting(e, WorkbookData))
		//		SetSpreadsheetDirty();
		//}

		//private void SpreadSheetOnCommandExecuted(object sender, SpreadsheetCommandExecutedEventArgs e)
		//	=> mCommands.OnCommandExecuted(e, WorkbookData);

		private void SpreadSheetOnActiveWorksheetChanged(object sender, SpreadsheetActiveWorksheetChangedEventArgs e)
			=> ActiveWorksheetName = e.NewValue.Name;

		private void SpreadSheetOnEditModeEntering(object sender, SpreadsheetEditModeEnteringEventArgs e)
			=> e.Cancel = IsReadOnly;

		private void SpreadsheetOnEditModeEntered(object sender, SpreadsheetEditModeEnteredEventArgs e)
			=> Spreadsheet.LostKeyboardFocus += SpreadsheetOnLostKeyboardFocus;

		private void SpreadSheetOnEditModeExited(object sender, SpreadsheetEditModeExitedEventArgs e)
		{
			Spreadsheet.LostKeyboardFocus -= SpreadsheetOnLostKeyboardFocus;

			if (mUndoItemNeeded)
			{
				//PushCurrentUndoItem();
				mUndoItemNeeded = false;
			}

			var activeWorksheet = (sender as XamSpreadsheet)?.ActiveWorksheet;

			if (activeWorksheet == null)
				return;

			CreateRowIfNotExists(activeWorksheet.Name, e.Cell.Row);
		}

		private void SpreadsheetOnLostKeyboardFocus(object sender, KeyboardFocusChangedEventArgs e)
		{
			// Don't exit edit mode when jumping between cell and formula bar
			if (e.NewFocus is Infragistics.Controls.Grids.Primitives.FormulaBarControl
				|| e.NewFocus is Infragistics.Controls.Grids.Primitives.FormulaBarTextEditorControl
				|| e.NewFocus is Infragistics.Controls.Grids.Primitives.SpreadsheetCellEditor)
				return;

			// Avoids getting here a second time causing the ExecuteCommand to throw a NullReferenceException
			Spreadsheet.LostKeyboardFocus -= SpreadsheetOnLostKeyboardFocus;

			// Update cell value to include it when Save was clicked
			Spreadsheet.ExecuteCommand(SpreadsheetCommandType.ExitEditModeAndUpdateActiveCell, null);
		}

		private void CreateRowIfNotExists(string sheetName, int row)
			=> _ = WorkbookData[sheetName].Rows[row];

		private object GetCellValue(SpreadsheetCell sourceCel)
			=> Spreadsheet.ActiveWorksheet.Rows[sourceCel.Row].Cells[sourceCel.Column].Value;

		private bool IsDropdownCell(WorksheetCell cell)
			=> cell.DataValidationRule is ListDataValidationRule;

		//private void AddDropdownCellUpdateIntoHistory()
		//{
		//	if (IsManualCellUpdate())
		//		PushCurrentUndoItem();
		//}

		//private bool IsManualCellUpdate()
		//	=> !mCommands.IsCommandExecuting;

		//private void PushCurrentUndoItem()
		//	=> mUndoHistory.PushCurrentUndoItem();

		private void SetSpreadsheetDirty()
			=> IsSpreadsheetDirty = true;
	}
}