﻿// <copyright company="PEER Group GmbH">
//    Copyright (c) PEER Group GmbH. All rights reserved.
// </copyright>

using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;
using EafClient.Core.Dispatchers;
using Infragistics.Controls.Menus;

namespace EafClient.Core.Behaviors
{
	/// <summary>
	/// The behavior for a XamDataTree that allows for binding to the single click command.
	/// </summary>
	public class XamDataTreeSingleClickCommandBehavior : Behavior<XamDataTree>
	{
		#region MouseEventTarget references

		/// <summary> A reference to the last <see cref="XamDataTreeNode"/> above which
		/// the left mouse button has been pushed. </summary>
		private XamDataTreeNode mLeftMouseDownTreeNode;

		/// <summary> A reference to the last <see cref="XamDataTreeNode"/> above which
		/// the left mouse button has been released. </summary>
		private XamDataTreeNode mLeftMouseUpTreeNode;

		/// <summary> A reference to the last <see cref="XamDataTreeNode"/> above which
		/// the right mouse button has been pushed. </summary>
		private XamDataTreeNode mRightMouseDownTreeNode;

		/// <summary> A reference to the last <see cref="XamDataTreeNode"/> above which
		/// the right mouse button has been released. </summary>
		private XamDataTreeNode mRightMouseUpTreeNode;

		#endregion MouseEventTarget references

		#region Constructors

		/// <summary> Initializes a new instance of the <see cref="XamDataTreeSingleClickCommandBehavior"/> class.
		/// </summary>
		public XamDataTreeSingleClickCommandBehavior() => IsNoSelectionAllowed = false;
		#endregion

		#region Dependency Properties

		/// <summary> The selected item dependency property. </summary>
		[System.Diagnostics.CodeAnalysis.SuppressMessage(
			"PeerGroup.StyleCop.PrefixForStaticFields", "*", Justification = "Public dependency property")]
		public static readonly DependencyProperty SingleClickCommandProperty =
			DependencyProperty.Register(
				"SingleClickCommand",
				typeof(ICommand),
				typeof(XamDataTreeSingleClickCommandBehavior));
		#endregion

		#region Properties

		/// <summary> Gets or sets the dependency property that contains the command that should be executed
		/// on left mouse click. </summary>
		public ICommand SingleClickCommand
		{
			get => (ICommand)GetValue(SingleClickCommandProperty);
			set => SetValue(SingleClickCommandProperty, value);
		}

		/// <summary> Gets or sets a value indicating whether it is allow to do not select any nodes. </summary>
		public bool IsNoSelectionAllowed { get; set; }
		#endregion

		#region Override Methods

		/// <summary> Called after the behavior is attached to an AssociatedObject. </summary>
		/// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
		protected override void OnAttached()
		{
			base.OnAttached();

			AssociatedObject.MouseLeftButtonDown += OnLeftMouseDown;
			AssociatedObject.MouseLeftButtonUp += OnLeftMouseUp;
			AssociatedObject.MouseRightButtonDown += OnRightMouseDown;
			AssociatedObject.MouseRightButtonUp += OnRightMouseUp;
			AssociatedObject.MouseDoubleClick += OnMouseDoubleClick;
		}

		/// <summary>
		/// Called when the behavior is being detached from its AssociatedObject, but before it has actually occurred.
		/// </summary>
		/// <remarks>Override this to unhook functionality from the AssociatedObject.</remarks>
		protected override void OnDetaching()
		{
			base.OnDetaching();

			AssociatedObject.MouseLeftButtonDown -= OnLeftMouseDown;
			AssociatedObject.MouseLeftButtonUp -= OnLeftMouseUp;
			AssociatedObject.MouseRightButtonDown -= OnRightMouseDown;
			AssociatedObject.MouseRightButtonUp -= OnRightMouseUp;
			AssociatedObject.MouseDoubleClick -= OnMouseDoubleClick;
		}
		#endregion

		#region MouseEvent Handler

		/// <summary> Handles the left mouse clicked event with respect to a tree. This event is being fired if the
		/// left mouse button is pushed down and released upwards above the same tree object. </summary>
		/// <param name="sourceNode">The <see cref="XamDataTreeNode"/> object
		/// over which the left mouse click was performed.</param>
		/// <param name="args">The <see cref="MouseButtonEventArgs"/> instance, that
		/// was forwarded from the left mouse down event, containing the event data.</param>
		private void OnLeftMouseClicked(XamDataTreeNode sourceNode, MouseButtonEventArgs args)
		{
			if (IsAnyMultiSelectKeyDown())
				return;

			if (sourceNode == null)
				return;

			DeselectAllNodes();
			sourceNode.Select();
			ExecuteSingleClickCommand(sourceNode.Data);
		}

		/// <summary> Handles the left mouse down event. This event is being fired if the left mouse button is
		/// being pushed. </summary>
		/// <param name="sender"> The sender of the left mouse button down event. </param>
		/// <param name="args"> The <see cref="MouseButtonEventArgs"/> instance containing the
		/// event data. </param>
		protected virtual void OnLeftMouseDown(object sender, MouseButtonEventArgs args)
		{
			mLeftMouseDownTreeNode = args.GetSourceXamDataTreeNode();
		}

		/// <summary> Handles the left mouse up event. This event is being fired if the left mouse button is
		/// being released. </summary>
		/// <param name="sender"> The sender of the left mouse up event. </param>
		/// <param name="args"> The <see cref="MouseButtonEventArgs"/> instance containing
		/// the event data. </param>
		private void OnLeftMouseUp(object sender, MouseButtonEventArgs args)
		{
			mLeftMouseUpTreeNode = args.GetSourceXamDataTreeNode();

			if (mLeftMouseUpTreeNode != null && mLeftMouseDownTreeNode != null)
			{
				if (mLeftMouseDownTreeNode == mLeftMouseUpTreeNode)
					OnLeftMouseClicked(mLeftMouseUpTreeNode, args);
			}
			else
				DeselectAllNodes();
		}

		/// <summary> Handles the right mouse down event. This event is being fired if the right mouse
		/// button is being pushed. </summary>
		/// <param name="sender"> The sender of the right mouse down event. </param>
		/// <param name="args"> The <see cref="MouseButtonEventArgs"/> instance containing
		/// the event data. </param>
		protected virtual void OnRightMouseDown(object sender, MouseButtonEventArgs args)
		{
			var multipleSelection = AreSelectedMultipleNodes();
			if (multipleSelection)
				return;

			mRightMouseDownTreeNode = args.GetSourceXamDataTreeNode();
			if (mRightMouseDownTreeNode != null)
			{
				DeselectAllNodes();
				mRightMouseDownTreeNode.Select();
			}
		}

		/// <summary> Deselects all nodes within the <see cref="XamDataTree"/>. </summary>
		private void DeselectAllNodes()
		{
			AssociatedObject.SelectionSettings.SelectedNodes.Clear();
			AssociatedObject.ActiveNode = null;
		}

		/// <summary>
		/// Determines whether multiple tree nodes are simultaneously selected.
		/// </summary>
		/// <returns>True if multiple nodes are simultaneously selected.</returns>
		private bool AreSelectedMultipleNodes()
		{
			return AssociatedObject.SelectionSettings.SelectedNodes.Count > 1;
		}

		/// <summary> Handles the right mouse up event. This event is being fired if the right mouse button is
		/// being released. </summary>
		/// <param name="sender"> The sender of the right mouse up event. </param>
		/// <param name="args"> The <see cref="MouseButtonEventArgs"/> instance containing
		/// the event data. </param>
		private void OnRightMouseUp(object sender, MouseButtonEventArgs args)
		{
			mRightMouseUpTreeNode = args.GetSourceXamDataTreeNode();
		}

		/// <summary> Handles the double click events. </summary>
		/// <param name="sender">The sender of the right mouse up event.</param>
		/// <param name="e"> The <see cref="MouseButtonEventArgs"/> instance containing
		/// the event data. </param>
		private void OnMouseDoubleClick(object sender, MouseButtonEventArgs e)
		{
			mLeftMouseDownTreeNode = e.GetSourceXamDataTreeNode();
			e.Handled = true;
		}
		#endregion MouseEventHandler

		#region Private Methods

		/// <summary> Checks whether any of the multi-select keys Left-CTRL, Right-CTRL, Left-SHIFT
		/// or Right-SHIFT are currently down. </summary>
		/// <returns>TRUE if any of the multi-select keys are down.</returns>
		private bool IsAnyMultiSelectKeyDown()
		{
			if (Keyboard.IsKeyDown(Key.LeftCtrl) ||
				Keyboard.IsKeyDown(Key.RightCtrl) ||
				Keyboard.IsKeyDown(Key.LeftShift) ||
				Keyboard.IsKeyDown(Key.RightShift))
			{
				return true;
			}

			return false;
		}

		/// <summary> Determines whether the source node has already been selected. </summary>
		/// <param name="sourceNode">The source node.</param>
		/// <returns>True if the source node has already been selected.</returns>
		private bool IsSourceNodeAlreadySelected(XamDataTreeNode sourceNode)
		{
			return AssociatedObject.SelectionSettings.SelectedNodes.Contains(sourceNode);
		}

		/// <summary> Executes the SingleClickCommand command by validating the command on the first place. </summary>
		/// <param name="data">Node data used by the execution of the command.</param>
		private void ExecuteSingleClickCommand(object data)
		{
			if (SingleClickCommand == null)
				return;

			if (SingleClickCommand.CanExecute(data))
				RootDispatcher.Current.Invoke(() => SingleClickCommand.Execute(data));
		}
		#endregion
	}
}
