﻿// <copyright company="PEER Group GmbH">
//    Copyright (c) PEER Group GmbH. All rights reserved.
// </copyright>

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Microsoft.Practices.Prism.Regions;

namespace SpreadsheetOwnerErrorSampleV3.Core.Regions
{
	/// <summary> Additional functionality for region navigation. </summary>
	public static class NavigationExtensions
	{
		#region IRegionManager Extensions
		
		public static void RequestNavigate(this IRegionManager regionManager, string regionName, string target,
			string parameterName, object parameterValue)
		{
			var parameters = new Dictionary<string, object>
				{
					{ parameterName, parameterValue }
				};
			RequestNavigate(regionManager, regionName, target, parameters);
		}
		
		public static void RequestNavigate(this IRegionManager regionManager, string regionName, string target,
			IDictionary<string, object> navigationParameters)
		{
			RequestNavigate(regionManager, regionName, new Uri(target, UriKind.RelativeOrAbsolute),
				nr => { }, navigationParameters);
		}

		public static void RequestNavigate(this IRegionManager regionManager, string regionName, string target,
			Action<NavigationResult> navigationCallback, IDictionary<string, object> navigationParameters)
		{
			RequestNavigate(regionManager, regionName, new Uri(target, UriKind.RelativeOrAbsolute),
				navigationCallback, navigationParameters);
		}
		
		public static void RequestNavigate(this IRegionManager regionManager, string regionName, Uri target,
			Action<NavigationResult> navigationCallback, IDictionary<string, object> navigationParameters)
		{
			if (regionManager == null)
				throw new ArgumentNullException(nameof(regionManager));
			var region = regionManager.Regions[regionName];
			region.RequestNavigate(target, navigationCallback, navigationParameters);
		}
		
		public static void RemoveFromRegion(this IRegionManager regionManager, object viewModel)
		{
			if (regionManager == null)
				throw new ArgumentNullException(nameof(regionManager));

			foreach (var region in regionManager.Regions)
			{
				var views = region.Views.Where(v => IsViewTarget(v, viewModel)).ToList();
				views.ForEach(x => region.Remove(x));
			}
		}
		
		public static void RemoveFromRegion(this IRegionManager regionManager, Predicate<object> isViewTarget)
		{
			if (regionManager == null)
				throw new ArgumentNullException(nameof(regionManager));

			foreach (var region in regionManager.Regions)
			{
				var views = region.Views.Where(v => isViewTarget(v)).ToList();
				views.ForEach(x => region.Remove(x));
			}
		}
		public static void RemoveAll(this IRegion region, params Type[] exclude)
		{
			if (region == null)
				throw new ArgumentNullException(nameof(region));

			foreach (var view in region.Views)
			{
				if (exclude.Contains(view.GetType()))
					continue;

				region.Deactivate(view);
				region.Remove(view);
			}
		}

		public static List<object> GetActiveViews(this IRegionManager regionManager, string regionName)
		{
			List<object> result = new List<object>();

			var views = regionManager.Regions[regionName].Views;

			if (views == null)
				return result;

			var activeViews = views.OfType<FrameworkElement>().Where(x => x.IsVisible);

			foreach (var view in activeViews)
			{
				if (view == null)
					continue;

				result.Add(view.DataContext);
			}

			return result;
		}
		
		public static List<T> GetObjects<T>(this IRegionManager regionManager)
		{
			var result = new List<T>();

			foreach (var region in regionManager.Regions)
			{
				foreach (var item in region.Views)
				{
					var el = item as FrameworkElement;
					if (el == null || el.DataContext == null)
						continue;

					if (el.DataContext is T)
					{
						result.Add((T)el.DataContext);
					}
				}
			}

			return result;
		}
		#endregion

		#region IRegion Extensions
		
		public static void RequestNavigate(this IRegion region, Uri target, Action<NavigationResult> navigationCallback,
			IDictionary<string, object> navigationParameters)
		{
			if (region == null)
				return;

			region.Context = navigationParameters;
			region.RequestNavigate(target, navigationCallback);
		}
		
		public static void UpdateActiveTab(this IRegionManager regionManager, string regionName)
		{
			if (regionManager == null)
				return;

			var region = regionManager.Regions[regionName];
			var activeViews = region.ActiveViews;

			// Currently support only one active view to be updated...
			if (activeViews == null || activeViews.Count() != 1)
				return;

			foreach (var view in activeViews)
			{
				var uri = new Uri(view.GetType().Name, UriKind.RelativeOrAbsolute);
				region.RequestNavigate(uri);
			}
		}
		#endregion

		#region Navigation Context
		public static IDictionary<string, object> GetParameters(this NavigationContext context)
		{
			if (context == null)
				return new Dictionary<string, object>();
			if (context.NavigationService == null || context.NavigationService.Region == null)
				return new Dictionary<string, object>();

			if (context.NavigationService.Region.Context is IDictionary<string, object>)
				return context.NavigationService.Region.Context as IDictionary<string, object>;

			return new Dictionary<string, object>();
		}
		
		public static T GetParameter<T>(this NavigationContext navigationContext, string parameterName)
		{
			if (navigationContext == null)
				throw new ArgumentNullException(nameof(navigationContext));
			if (parameterName == null)
				throw new ArgumentNullException(nameof(parameterName));

			var parameters = navigationContext.GetParameters();
			if (parameters.TryGetValue(parameterName, out var value))
				return (T)value;

			throw new ArgumentOutOfRangeException(nameof(parameterName),
				$"Navigation parameter '{parameterName}' does not exist");
		}
		
		public static bool ContainsParameter(this NavigationContext navigationContext, string parameterName)
		{
			if (navigationContext == null)
				throw new ArgumentNullException(nameof(navigationContext));
			if (parameterName == null)
				throw new ArgumentNullException(nameof(parameterName));

			var parameters = navigationContext.GetParameters();
			return parameters.ContainsKey(parameterName);
		}
		#endregion Navigation Context

		#region Private Methods
		
		private static bool IsViewTarget(object view, object target)
		{
			var frameworkElement = view as FrameworkElement;
			if (frameworkElement == null)
				return false;

			return frameworkElement.DataContext == target;
		}
		#endregion
	}
}
