﻿// <copyright company="PEER Group GmbH">
// Copyright (c) PEER Group GmbH. All rights reserved.
// </copyright>

using System;
using System.Linq;
using Infragistics.Documents.Excel;
using Microsoft.Practices.ObjectBuilder2;

namespace SpreadsheetOwnerErrorSampleV3.SpreadsheetControl
{
	/// <summary>A basic <see cref="ISheetStyle"/> implementation.</summary>
	internal class SheetStyle : ISheetStyle
	{
		/// <summary>The font height multiplier to use for calculation based on point font sizes.</summary>
		private const int FONT_HEIGHT_MULTIPLIER = 20;
		private const double PX_TO_PT_MULTIPLIER = 0.74999943307122;

		/// <summary>Holds the font name.</summary>
		private readonly string mFontName;

		/// <summary>Gets the calculated font height in twips (1/20 of a point).</summary>
		private readonly int mFontHeightTwips;

		/// <summary>Initializes a new instance of the <see cref="SheetStyle"/> class.</summary>
		/// <param name="fontName">The font name.</param>
		/// <param name="fontSizePx">The font size in px.</param>
		public SheetStyle(string fontName, double fontSizePx)
		{
			mFontName = fontName;
			mFontHeightTwips = ConvertToFontHeight(fontSizePx);
		}

		/// <inheritdoc/>
		public Workbook ApplyTo(Workbook workbook)
		{
			ApplyTo(workbook.Styles);

			workbook.Worksheets.ForEach(
				sheet => sheet.Rows.ForEach(
					row => row.Cells.Where(cell => cell.HasCellFormat).ForEach(
						cell => ApplyTo(cell.CellFormat))));

			return workbook;
		}

		/// <summary>Converts the given font size in px to a font height value in twips (1/20 of a point).</summary>
		/// <param name="fontSizePx">The font size in px.</param>
		/// <returns>The calculated font height in twips (1/20 of a point).</returns>
		private static int ConvertToFontHeight(double fontSizePx) =>
			Convert.ToInt32(fontSizePx * PX_TO_PT_MULTIPLIER * FONT_HEIGHT_MULTIPLIER);

		private void ApplyTo(WorkbookStyleCollection styleCollection)
		{
			ApplyTo(styleCollection.NormalStyle);
			styleCollection.ForEach(ApplyTo);
		}

		private void ApplyTo(WorkbookStyle style) => ApplyTo(style.StyleFormat.Font);

		private void ApplyTo(IWorksheetCellFormat cellFormat)
			=> ApplyTo(cellFormat.Font);

		private void ApplyTo(IWorkbookFont font)
		{
			font.Name = mFontName;
			font.Height = mFontHeightTwips;
		}
	}
}