﻿// <copyright file="SpreadsheetRows.cs" company="PEER Group GmbH">
// Copyright (c) PEER Group GmbH. All rights reserved.
// </copyright>

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Infragistics.Documents.Excel;
using Microsoft.Practices.ObjectBuilder2;
using SpreadsheetOwnerErrorSampleV3.Interfaces;

namespace SpreadsheetOwnerErrorSampleV3.SpreadsheetControl
{
	internal class SpreadsheetRows : IList<IWorksheetRow>
	{
		private readonly SpreadsheetRowFactory mRowFactory;
		private readonly Worksheet mWorksheet;
		private readonly List<IWorksheetRow> mRows = new List<IWorksheetRow>();

		private Hashtable mRowsByIndex;
		private bool mIsRangeUpdateAllowed = true;
		private int? mActualColumnsCount;
		private int mActualRowsCount;

		public SpreadsheetRows(Worksheet worksheet)
		{
			mWorksheet = worksheet;
			mRowFactory = new SpreadsheetRowFactory();

			mWorksheet.Rows.ForEach(r => mRows.Add(mRowFactory.Create(r)));
			InitializeRowsByIndex();
		}

		public void EnterBulkRowInsert()
			=> mIsRangeUpdateAllowed = false;

		public void ExitBulkRowInsert()
		{
			mIsRangeUpdateAllowed = true;
			UpdateWorksheetSetup();
		}

		public IWorksheetRow this[int index]
		{
			get => GetRowOrCreateIfNotExist(index);
			set => throw new NotSupportedException();
		}

		public int Count => mRows.Max(row => row.Index) + 1;

		public bool IsReadOnly => true;

		public void Add(IWorksheetRow item)
			=> throw new InvalidOperationException("Use this[index] operator instead.");

		public void Clear()
		{
			mRows.Clear();
			mRowsByIndex = new Hashtable();
		}

		public bool Contains(IWorksheetRow item)
			=> mRows.Contains(item);

		public void CopyTo(IWorksheetRow[] array, int arrayIndex)
			=> mRows.CopyTo(0, array, arrayIndex, array.Length);

		public IEnumerator<IWorksheetRow> GetEnumerator()
			=> mRows
				.Where(r => r.Index > -1)
				.OrderBy(r => r.Index)
				.GetEnumerator();

		public int IndexOf(IWorksheetRow item)
			=> mRows.IndexOf(item);

		public void Insert(int index, IWorksheetRow item)
			=> throw new InvalidOperationException("Use this[index] operator instead.");

		public bool Remove(IWorksheetRow item)
			=> throw new InvalidOperationException("Remove is not allowed.");

		public void RemoveAt(int index)
			=> throw new InvalidOperationException("RemoveAt is not allowed.");

		IEnumerator IEnumerable.GetEnumerator()
			=> GetEnumerator();

		private IWorksheetRow GetRowOrCreateIfNotExist(int index)
		{
			while (true)
			{
				// Try to find row in the hash table.
				if (!(mRowsByIndex[index] is IWorksheetRow foundRow))
					return AddNewRow(mWorksheet.Rows[index]);

				// Check if the hash table has valid row indexes.
				if (foundRow.Index == index)
					return foundRow;

				// Reset invalid hash table to get updated row indexes.
				InitializeRowsByIndex();

				// Try to get (or add) a row in the next iteration.
			}
		}

		private void InitializeRowsByIndex()
		{
			mRowsByIndex = new Hashtable();
			mRows
				.Where(r => r.Index > -1)
				.ForEach(r => mRowsByIndex.Add(r.Index, r));
		}

		private IWorksheetRow AddNewRow(WorksheetRow row)
		{
			var spreadsheetRow = mRowFactory.Create(row);
			mRows.Add(spreadsheetRow);
			mRowsByIndex.Add(spreadsheetRow.Index, spreadsheetRow);

			if (mIsRangeUpdateAllowed)
				UpdateWorksheetSetup();

			return spreadsheetRow;
		}

		private void UpdateWorksheetSetup()
		{
			UpdateActualRowsCount();
			UpdateRangesForParentWorksheet();
			UpdateFilterForParentWorksheet();
			UpdateSortRegionForParentWorksheet();
		}

		private void UpdateActualRowsCount()
			=> mActualRowsCount = mWorksheet.Rows.Max(r => r.Index);

		private int GetActualColumnsCount()
			=> mActualColumnsCount ??= mWorksheet.Rows[0].Cells.Count();

		private void UpdateRangesForParentWorksheet()
		{
			// Temporary fix for the dropdown list issue in the spreadsheet control.
			foreach (var range in mWorksheet.Workbook.NamedReferences
				.Where(r => r.ReferencedRegion != null && r.ReferencedRegion.Worksheet.Equals(mWorksheet)))
			{
				var region = range.ReferencedRegion;
				var updatedRegion = new WorksheetRegion(mWorksheet, region.FirstRow, region.FirstColumn, mActualRowsCount,
					region.LastColumn);

				range.SetFormula($"={updatedRegion.ToString(CellReferenceMode.R1C1, true)}", CellReferenceMode.R1C1);
			}
		}

		private void UpdateFilterForParentWorksheet()
		{
			if (mActualRowsCount < 1 || GetActualColumnsCount() < 2)
				return;

			var filterRegion = new WorksheetRegion(mWorksheet, 0, 0, mActualRowsCount, GetActualColumnsCount() - 1);
			mWorksheet.FilterSettings.SetRegion(filterRegion.ToString(CellReferenceMode.R1C1, true),
				CellReferenceMode.R1C1);
		}

		private void UpdateSortRegionForParentWorksheet()
		{
			if (GetActualColumnsCount() < 1)
				return;

			var sortRegion = new WorksheetRegion(mWorksheet, 0, 0, 0, GetActualColumnsCount() - 1);
			mWorksheet.SortSettings.SetRegion(sortRegion.ToString(CellReferenceMode.R1C1, true), CellReferenceMode.R1C1);
		}
	}
}