﻿// <copyright file="SpreadsheetWorkbook.cs" company="PEER Group GmbH">
// Copyright (c) PEER Group GmbH. All rights reserved.
// </copyright>

using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Infragistics.Controls.Grids;
using Infragistics.Documents.Excel;
using SpreadsheetOwnerErrorSampleV3.Interfaces;

namespace SpreadsheetOwnerErrorSampleV3.SpreadsheetControl
{
	internal class SpreadsheetWorkbook : IWorkbook
	{
		#region Private Members

		private readonly Workbook mWorkbook;
		private readonly Dictionary<string, IWorksheet> mSpreadSheets = new Dictionary<string, IWorksheet>();
		private readonly SpreadsheetFactory mSpreadsheetFactory;

		#endregion

		#region Public Members

		public IWorksheet this[string name]
		{
			get
			{
				CreateSheetIfNotExists(name);
				return mSpreadSheets[name];
			}
		}

		#endregion

		#region Constructors

		/// <summary>Initializes a new instance of the <see cref="SpreadsheetWorkbook"/> class.</summary>
		/// <param name="workbook">The initial <see cref="Workbook"/>.</param>
		public SpreadsheetWorkbook(Workbook workbook)
		{
			mWorkbook = workbook;

			mSpreadsheetFactory = new SpreadsheetFactory(mWorkbook);

			PopulateSpreadSheets();
		}

		/// <summary>Initializes a new instance of the <see cref="SpreadsheetWorkbook"/> class.</summary>
		/// <param name="workbook">The initial <see cref="Workbook"/>.</param>
		/// <param name="sheetStyle">The sheet style to apply.</param>
		public SpreadsheetWorkbook(Workbook workbook, ISheetStyle sheetStyle)
			: this(sheetStyle?.ApplyTo(workbook) ?? workbook)
		{
		}

		#endregion

		#region Public Methods

		public void DeleteActiveWorksheet()
		{
			var selectedWorksheet = mWorkbook.WindowOptions.SelectedWorksheet;
			var index = selectedWorksheet.Index;
			DeleteWorksheet(selectedWorksheet);
			SelectPreviousWorksheet(index);
		}

		public void RenameWorksheet(string newName, string oldName)
		{
			if (newName == oldName)
				return;

			// rename Spreadsheet
			mSpreadSheets[newName] = mSpreadSheets[oldName];
			mSpreadSheets.Remove(oldName);
			mSpreadSheets[newName].Name = newName;
		}

		/// <inheritdoc />
		public void SetActiveWorksheet(string name)
			=> mWorkbook.WindowOptions.SelectedWorksheet =
				mWorkbook.Worksheets.FirstOrDefault(@ref => @ref.Name == name);

		/// <inheritdoc />
		public IWorksheet GetActiveWorksheet() => this[mWorkbook.WindowOptions.SelectedWorksheet.Name];

		public IEnumerable<string> GetSheetNames()
			=> mWorkbook.Worksheets.Select(@ref => @ref.Name);

		public void ApplyTo(XamSpreadsheet spreadsheetControl)
			=> spreadsheetControl.Workbook = mWorkbook;

		public void Save(string fileName)
			=> mWorkbook.Save(fileName);

		public IEnumerator<IWorksheet> GetEnumerator() => mSpreadSheets
			.Select(s => s.Value)
			.GetEnumerator();

		#endregion

		#region Explicit Interface Implementations

		IEnumerator IEnumerable.GetEnumerator() => GetEnumerator();

		#endregion

		#region Private Methods
		private void SelectPreviousWorksheet(int index)
		{
			while (index > 0)
			{
				index--;
				if (index == 0 || mWorkbook.Worksheets[index].DisplayOptions.Visibility == WorksheetVisibility.Visible)
					break;
			}

			mWorkbook.WindowOptions.SelectedWorksheet = mWorkbook.Worksheets[index];
		}

		private void DeleteWorksheet(Worksheet worksheet)
		{
			mWorkbook.Worksheets.Remove(worksheet);
			mSpreadSheets.Remove(worksheet.Name);
		}

		private void PopulateSpreadSheets()
		{
			foreach (var item in GetSheetNames())
			{
				CreateSheetIfNotExists(item);
			}
		}

		private void CreateSheetIfNotExists(string sheetName)
		{
			if (!mSpreadSheets.ContainsKey(sheetName))
				mSpreadSheets[sheetName] = mSpreadsheetFactory.CreateWorksheet(sheetName);
		}

		#endregion
	}
}
